<?php
/************************************************************
* author       : Ad Bakker
* plugin       : Signature Images
* last_mod     : 07.29.2015 (July 29th 2015)
* version      : 2.1 (max image width & height for usergroups 2.0
*                     max image size in kB 2.1)
*************************************************************/
if(!defined('IN_MYBB'))
{
    die("This file cannot be accessed");
}

function sigImg_info ()
{
    return array (
        "name"          => "Signature Images",
        "description"   => "Set the maximum size of images in a signature and the maximum number of images in signatures per user group.",
        'website'       => "",
        'author'        => "Ad Bakker", 
        'authorsite'    => "",
        'version'       => "1.0",
        'guid'          => "",
        'compatibility' => "18*",
    );
}

function sigImg_install()
{
	global $db, $lang, $cache;
/*
 * Database table usergroups
 * Add etra columns maxsigimages, maximgwidth and maximgheight
 */
	if (!$db->field_exists("maxsigimages", "usergroups"))
	{
		$db->add_column("usergroups", "maxsigimages", "smallint(2) default 0");
		$db->add_column("usergroups", "maximgwidth", "smallint(5) default 0");
		$db->add_column("usergroups", "maximgheight", "smallint(5) default 0");
		$db->add_column("usergroups", "maximgsize", "smallint(5) default 0");
		// rebuild usergroups cache
		$cache->update_usergroups();
	}
}

function sigImg_uninstall()
{
	global $db, $cache;
	
	// Remove columns maxsigimages, maximgwidth and maximgheight from database table usergroups
	$db->drop_column("usergroups", "maxsigimages");
	$db->drop_column("usergroups", "maximgwidth");
	$db->drop_column("usergroups", "maximgheight");
	$db->drop_column("usergroups", "maximgsize");
	$cache->update_usergroups();
}

function sigImg_is_installed()
{
    global $db;
    if($db->field_exists("maxsigimages", "usergroups"))
    {
        return true;
    }
    return false;
}

$plugins->add_hook("usercp_start", "sigImg_set_max_images");
function sigImg_set_max_images()
{
    global $mybb, $parser, $lang, $groupscache;
    
    if(!($mybb->input['action'] == "do_editsig" && $mybb->request_method == "post")) return;
	
	$mybb->settings['maxsigimages'] = $groupscache[$mybb->user['usergroup']]['maxsigimages'];
}


$plugins->add_hook("usercp_do_editsig_start", "sigImg_resize_images");
function sigImg_resize_images()
{
    global $mybb, $groupscache, $lang, $error;
/*
	if (isset($mybb->cookies['sigImg_message']))
	{
		my_unsetcookie('sigImg_message');
	}
*/
	if (isset($mybb->input['sigImg_message']))
	{
		unset ($mybb->input['sigImg_message']);
	}
	$lang->load(sigImg);
	$sig_message = "";
	$maximgwidth = $groupscache[$mybb->user['usergroup']]['maximgwidth'];
	$maximgheight = $groupscache[$mybb->user['usergroup']]['maximgheight'];
	$maximgsize = $groupscache[$mybb->user['usergroup']]['maximgsize'];
	$sig = $mybb->get_input('signature');
	$sig = str_ireplace("[IMG", "[JMG", $sig);
	$match_pattern = '#\[JMG(.*?)\[/IMG\]#is';
	preg_match_all($match_pattern,$sig,$matches,PREG_SET_ORDER);
	foreach ($matches as $found)
	{
		$img = preg_replace($match_pattern,"$1",$found[0]);
		if (substr($img, 0, 1) != "]")
		{
			$src = substr($img, strpos($img, "]")+1);
		}
		else
		{
			$src = substr($img, 1);
		}
		$fileSize = getRemoteFilesize($src);
		$imgsize = ceil($fileSize/1024);
		if ($imgsize > $maximgsize && $maximgsize > 0)
		{
			// this message is displayed later via the usercp_editsig_end hook
			if (isset($mybb->cookies['sigImg_message']))
			{
				$sig_message = $mybb->cookies['sigImg_message'];
				my_unsetcookie('sigImg_message');
			}
			$sig_message .= $lang->sprintf($lang->sigImg_size_error, $src, $imgsize, $maximgsize);
			my_setcookie("sigImg_message", "{$sig_message}");		
			
			$replacement = "[URL]".$src."[/URL]";
			$sig = preg_replace($match_pattern,$replacement,$sig,1);
			continue;
		}
		if (substr($img, 0, 1) != "]")
		{
			preg_match_all('#([\d]+)#', $img, $numbers);
			$width = $numbers[0][0];
			$height = $numbers[0][1];
			
			if ($height <= $mybb->settings['sigImg_max_height'] && $width <= $mybb->settings['sigImg_max_width'])
			{
//				$sig = preg_replace("#[JMG#i", "[IMG", $sig, 1); // unexplainable why this does not work
				$sig = replace_once ("[JMG", "[IMG", $sig); //workaround
				continue;
			}
			$scale_height = $maximgheight/$height;
			$scale_width = $maximgwidth/$width;
			if ($scale_height <= $scale_width)
			{
				$height_scaled = $maximgheight;
				$width_scaled = $width*$maximgheight/$height;
			}
			else
			{
				$width_scaled = $maximgwidth;
				$height_scaled = $height*$maximgwidth/$width;
			}
		}
		else
		{
			$img_size = getimagesize ($src);
			$width_img = $img_size[0];
			$height_img = $img_size[1];
			if ($height_img <= $maximgheight && $width_img <= $maximgwidth)
			{
//				$sig = preg_replace("#[JMG#i", "[IMG", $sig, 1); // unexplainable why this does not work
				$sig = replace_once ("[JMG", "[IMG", $sig); // workaround
				continue;
			}
			if ($maximgwidth/$width_img <= $maximgheight/$height_img)
			{
				$width_scaled = $maximgwidth;
				$height_scaled = $height_img*$maximgwidth/$width_img;;
			}
			else
			{
				$height_scaled = $maximgheight;
				$width_scaled = $width_img*$maximgheight/$height_img;
			}
		}
		$replacement = "[IMG=".(int)$width_scaled."x".(int)$height_scaled."]".$src."[/IMG]";
		$sig = preg_replace($match_pattern,$replacement,$sig,1);
	}
	$mybb->input['signature'] = $sig;
	$file = fopen ("zzz_debug", "w");
	fwrite ($file, "Signature = ".$sig);
	fclose ($file);
}

// this is a workaround for preg_replace error (why does it not work??, at least sometimes) 
function replace_once ($org, $rep, $string)
{
	$pos = stripos($string, $org);
	if ($pos === false) return $string;
	$len = strlen($org);
	$before = "";
	if ($pos > 0) $before = substr($string, 0, $pos);
	$after = "";
	if ($pos + $len < strlen($string)) $after = substr($string, $pos + $len);
	return $before.$rep.$after;
}

$plugins->add_hook("usercp_editsig_end", "usercp_editsig_end");
function usercp_editsig_end()
{
	global $mybb, $headerinclude;

	if (isset($mybb->cookies['sigImg_message']))
	{
		$sig_message = $mybb->cookies['sigImg_message'];
		$headerinclude .= '<script type="text/javascript">alert("'.$sig_message.'")</script>';
		my_unsetcookie('sigImg_message');
	}
}

$plugins->add_hook("admin_user_groups_begin", "sigImg_setmximages");
function sigImg_setmximages()
{
	global $usergroup_permissions, $lang;
	
	$lang->load('sigImg');
	$usergroup_permissions['maxsigimages'] = 0;
	$usergroup_permissions['maximgwidth'] = 0;
	$usergroup_permissions['maximgheight'] = 0;
	$usergroup_permissions['maximgsize'] = 0;
}

$plugins->add_hook("admin_user_groups_edit_commit", "sigImg_updmximages");
function sigImg_updmximages()
{
	global $updated_group, $mybb;
	$updated_group['maxsigimages'] = $mybb->get_input('maxsigimages', MyBB::INPUT_INT);
	$updated_group['maximgwidth'] = $mybb->get_input('maximgwidth', MyBB::INPUT_INT);
	$updated_group['maximgheight'] = $mybb->get_input('maximgheight', MyBB::INPUT_INT);
	$updated_group['maximgsize'] = $mybb->get_input('maximgsize', MyBB::INPUT_INT);
}

$plugins->add_hook("admin_formcontainer_output_row", "sigImg_usergroup_misc_options");
function sigImg_usergroup_misc_options (&$pluginargs)
{
	global $lang, $mybb;
	if ($pluginargs['title'] == $lang->misc && strpos($pluginargs['content'], "<div class=\"group_settings_bit\">") !== false)
	{
		$pluginargs['content'] .= "<div class=\"group_settings_bit\">{$lang->max_images_sig}:<br /><small class=\"input\">{$lang->max_images_sig_desc}</small><br /><input type=\"number\" name=\"maxsigimages\" value=\"{$mybb->input['maxsigimages']}\" min=\"0\" class=\"text_input field50\" id=\"maxsigimages\" /></div>";
		
		$pluginargs['content'] .= "<div class=\"group_settings_bit\">{$lang->sigImg_max_width_title}:<br /><small class=\"input\">{$lang->sigImg_max_width_desc}</small><br /><input type=\"number\" name=\"maximgwidth\" value=\"{$mybb->input['maximgwidth']}\" min=\"0\" class=\"text_input field50\" id=\"maximgwidth\" /></div>";
		
		$pluginargs['content'] .= "<div class=\"group_settings_bit\">{$lang->sigImg_max_height_title}:<br /><small class=\"input\">{$lang->sigImg_max_height_desc}</small><br /><input type=\"number\" name=\"maximgheight\" value=\"{$mybb->input['maximgheight']}\" min=\"0\" class=\"text_input field50\" id=\"maximgheight\" /></div>";
		
		$pluginargs['content'] .= "<div class=\"group_settings_bit\">{$lang->sigImg_max_size_title}:<br /><small class=\"input\">{$lang->sigImg_max_size_desc}</small><br /><input type=\"number\" name=\"maximgsize\" value=\"{$mybb->input['maximgsize']}\" min=\"0\" class=\"text_input field50\" id=\"maximgsize\" /></div>";
	}
}

function getRemoteFilesize($url)
{
    $head = array_change_key_case(get_headers($url, 1));
    // content-length of download (in bytes), read from Content-Length: field
    $clen = isset($head['content-length']) ? $head['content-length'] : 0;
    // cannot retrieve file size, return "-1"
    if (!$clen) {
        return -1;
    }
    return $clen; // return size in bytes
}
